94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
142 lines
4.3 KiB
Python
142 lines
4.3 KiB
Python
import torch
|
|
import triton # type: ignore
|
|
import triton.language as tl # type: ignore
|
|
|
|
from sglang.multimodal_gen.runtime.platforms import current_platform
|
|
|
|
|
|
@triton.autotune(
|
|
configs=[
|
|
triton.Config({"BLOCK_HEADS": 1, "BLOCK_HS_HALF": 32}, num_warps=2),
|
|
triton.Config({"BLOCK_HEADS": 2, "BLOCK_HS_HALF": 32}, num_warps=2),
|
|
triton.Config({"BLOCK_HEADS": 4, "BLOCK_HS_HALF": 32}, num_warps=4),
|
|
triton.Config({"BLOCK_HEADS": 4, "BLOCK_HS_HALF": 64}, num_warps=4),
|
|
triton.Config({"BLOCK_HEADS": 8, "BLOCK_HS_HALF": 64}, num_warps=8),
|
|
],
|
|
key=["num_heads", "head_size"],
|
|
)
|
|
@triton.jit
|
|
def _rotary_embedding_kernel(
|
|
output_ptr,
|
|
x_ptr,
|
|
cos_ptr,
|
|
sin_ptr,
|
|
num_heads,
|
|
head_size,
|
|
num_tokens,
|
|
stride_out_bt,
|
|
stride_out_head,
|
|
stride_x_bt,
|
|
stride_x_head,
|
|
stride_cos_row,
|
|
stride_sin_row,
|
|
BLOCK_HEADS: tl.constexpr,
|
|
BLOCK_HS_HALF: tl.constexpr,
|
|
):
|
|
bt_idx = tl.program_id(0)
|
|
head_block_idx = tl.program_id(1)
|
|
token_idx = bt_idx % num_tokens
|
|
|
|
cos_row_ptr = cos_ptr + token_idx * stride_cos_row
|
|
sin_row_ptr = sin_ptr + token_idx * stride_sin_row
|
|
head_offsets = head_block_idx * BLOCK_HEADS + tl.arange(0, BLOCK_HEADS)
|
|
head_mask = head_offsets < num_heads
|
|
|
|
head_size_half = head_size // 2
|
|
x_row_ptrs = x_ptr + bt_idx * stride_x_bt + head_offsets[:, None] * stride_x_head
|
|
output_row_ptrs = (
|
|
output_ptr + bt_idx * stride_out_bt + head_offsets[:, None] * stride_out_head
|
|
)
|
|
|
|
for block_start in range(0, head_size_half, BLOCK_HS_HALF):
|
|
offsets_half = block_start + tl.arange(0, BLOCK_HS_HALF)
|
|
half_mask = offsets_half < head_size_half
|
|
mask = head_mask[:, None] & half_mask[None, :]
|
|
|
|
cos_vals = tl.load(cos_row_ptr + offsets_half, mask=half_mask, other=0.0)
|
|
sin_vals = tl.load(sin_row_ptr + offsets_half, mask=half_mask, other=0.0)
|
|
|
|
offsets_x1 = 2 * offsets_half
|
|
offsets_x2 = 2 * offsets_half + 1
|
|
|
|
x1_vals = tl.load(x_row_ptrs + offsets_x1[None, :], mask=mask, other=0.0)
|
|
x2_vals = tl.load(x_row_ptrs + offsets_x2[None, :], mask=mask, other=0.0)
|
|
|
|
x1_fp32 = x1_vals.to(tl.float32)
|
|
x2_fp32 = x2_vals.to(tl.float32)
|
|
cos_fp32 = cos_vals.to(tl.float32)[None, :]
|
|
sin_fp32 = sin_vals.to(tl.float32)[None, :]
|
|
o1_vals = tl.fma(-x2_fp32, sin_fp32, x1_fp32 * cos_fp32)
|
|
o2_vals = tl.fma(x1_fp32, sin_fp32, x2_fp32 * cos_fp32)
|
|
|
|
tl.store(
|
|
output_row_ptrs + offsets_x1[None, :],
|
|
o1_vals.to(x1_vals.dtype),
|
|
mask=mask,
|
|
)
|
|
tl.store(
|
|
output_row_ptrs + offsets_x2[None, :],
|
|
o2_vals.to(x2_vals.dtype),
|
|
mask=mask,
|
|
)
|
|
|
|
|
|
def apply_rotary_embedding(
|
|
x: torch.Tensor, cos: torch.Tensor, sin: torch.Tensor, interleaved: bool = False
|
|
) -> torch.Tensor:
|
|
output = torch.empty_like(x)
|
|
|
|
if x.dim() > 3:
|
|
bsz, num_tokens, num_heads, head_size = x.shape
|
|
else:
|
|
num_tokens, num_heads, head_size = x.shape
|
|
bsz = 1
|
|
|
|
assert head_size % 2 == 0, "head_size must be divisible by 2"
|
|
|
|
x_reshaped = x.view(bsz * num_tokens, num_heads, head_size)
|
|
output_reshaped = output.view(bsz * num_tokens, num_heads, head_size)
|
|
|
|
if interleaved and cos.shape[-1] == head_size:
|
|
cos = cos[..., ::2].contiguous()
|
|
sin = sin[..., ::2].contiguous()
|
|
else:
|
|
cos = cos.contiguous()
|
|
sin = sin.contiguous()
|
|
|
|
_rotary_embedding_kernel[
|
|
lambda META: (bsz * num_tokens, triton.cdiv(num_heads, META["BLOCK_HEADS"]))
|
|
](
|
|
output_reshaped,
|
|
x_reshaped,
|
|
cos,
|
|
sin,
|
|
num_heads,
|
|
head_size,
|
|
num_tokens,
|
|
output_reshaped.stride(0),
|
|
output_reshaped.stride(1),
|
|
x_reshaped.stride(0),
|
|
x_reshaped.stride(1),
|
|
cos.stride(0),
|
|
sin.stride(0),
|
|
)
|
|
|
|
return output
|
|
|
|
|
|
if current_platform.is_npu():
|
|
from .npu_fallback import apply_rotary_embedding_native
|
|
|
|
apply_rotary_embedding = apply_rotary_embedding_native
|
|
|
|
if current_platform.is_mps():
|
|
from .mps_fallback import apply_rotary_embedding_native
|
|
|
|
apply_rotary_embedding = apply_rotary_embedding_native
|
|
|
|
if current_platform.is_cpu():
|
|
from .torch_fallback import apply_rotary_embedding_native
|
|
|
|
apply_rotary_embedding = apply_rotary_embedding_native
|